/*
 *  RC5 transmitter v0.1
 *
 *  Created: 07.08.2014 22:31:24
 *  Author: Fabio
 *  License: Open source
 */ 


#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdbool.h>
#define F_CPU 8000000UL

#include <util/delay.h>

volatile uint16_t rc5Command =        0b00000000000000;		// Stores the entire RC5 Command
volatile uint16_t rc5Commandmask =    0b10000000000000;		// RC5 Command mask
	
uint16_t rc5Start =			 0b10000000000000;				// RC5 Start bit must be 1
uint16_t rc5Field =          0b01000000000000;				// RC5 Field bit choose the high or low Field of the RC5 Code. Could be 1 or 0.
uint16_t rc5Toggle =         0b00100000000000;				// RC5 Toggle bit chance his value even a RC5 Frame being sent
uint16_t rc5Address =        0b00010001000000;				// RC5 Address bits selects the device address, for communication with the selected devices
uint16_t rc5Code =           0b00000000000000;				// RC5 code bits that are used, to generated the next Code for the following RC5 Frame

uint16_t rc5StartBits =      0x2000;						// RC5 Start bits mask
uint16_t rc5FieldBits =      0x1000;						// RC5 Field bits mask
uint16_t rc5ToggleBits =     0x800;							// RC5 Toggle bits mask
uint16_t rc5AddressBits =    0x400;		     				// RC5 Address bits mask
uint16_t rc5CodeBits =       0x20;							// RC5 code bits mask


uint8_t rc5StartMSB =       13;								// RC5 most significant bit Start bit 
uint8_t rc5StartLSB =		13;								// RC5 last significant bit Start bit 
uint8_t rc5FieldMSB =       12;								// RC5 most significant bit Field bits 
uint8_t rc5FieldLSB =		12;								// RC5 last significant bit Field bits
uint8_t rc5ToggleMSB =		11;								// RC5 most significant bit Toggle bits 
uint8_t rc5ToggleLSB =		11;								// RC5 last significant bit Toggle bits
uint8_t rc5AddressMSB =		10;								// RC5 most significant bit Address bits 
uint8_t rc5AddressLSB =		6; 								// RC5 last significant bit Address bits
uint8_t rc5CodeMSB =		5; 								// RC5 most significant bit code bits 
uint8_t rc5CodeLSB =		0; 								// RC5 last significant bit code bits

int i;														// the index of the for-loops
	
	
// uint8_t myID = 1;
// uint8_t myRole = 21;
// uint8_t targetLocated = 41;


uint8_t white       = 0b111111;					//63		// RC5 Code white used to set next RC5 code
uint8_t red         = 0b111110;					//62		// RC5 Code red used to set next RC5 code
uint8_t green       = 0b111101;					//61		// RC5 Code green used to set next RC5 code
uint8_t blue        = 0b111100;					//60		// RC5 Code blue used to set next RC5 code
uint8_t yellow      = 0b111011;					//59		// RC5 Code yellow used to set next RC5 code
uint8_t purple      = 0b111010;					//58		// RC5 Code purple used to set next RC5 code
uint8_t turquoise   = 0b111001;					//57		// RC5 Code turquoise used to set next RC5 code
uint8_t black       = 0b111000;					//56		// RC5 Code black used to set next RC5 code
	
#define selectWhite      0b111111				//63		// RC5 Code selectWhite used to select next RC5 code
#define selectRed        0b111110				//62		// RC5 Code selectRed used to select next RC5 code
#define selectGreen      0b111101				//61		// RC5 Code selectGreen used to select next RC5 code
#define selectBlue       0b111100				//60		// RC5 Code selectYellow used to select next RC5 code
#define selectYellow     0b111011				//59		// RC5 Code selectYellow used to select next RC5 code
#define selectPurple     0b111010				//58		// RC5 Code selectPurple used to select next RC5 code
#define selectTurquoise  0b111001				//57		// RC5 Code selectTurquoise used to select next RC5 code
#define selectBlack		 0b111000				//56		// RC5 Code selectBlack used to select next RC5 code




volatile uint8_t rc5CommandSent = 0;						// Variable to check if the RC5 Frame is sent complete				
volatile uint8_t compareMatchCounter = 0;					// Variable to determine the transmit half bit time
volatile uint8_t sendRc5Command = 0;						// Variable to enable the next RC5 Transmit

volatile uint8_t rc5BitTransmit = 0;						// Checks if one bit is transmitted
volatile uint8_t selectRc5Code = 0;							// Variable to choice the next RC5 code




uint8_t generateRc5Code(uint8_t genRc5Code)					// Method to generate the RC5 Code
{
 	uint8_t rc5CodeBits =       0x20;						// RC5 code bits mask
 	
 	uint8_t rc5CodeMSB =		5;							// RC5 most significant bit code bits
 	uint8_t rc5CodeLSB =		0;							// RC5 last significant bit code bits
	
	if(selectRc5Code > 55)									// checks the selecting variable
	{
		selectRc5Code--;									// decrements the variable each rc5 transfer
	}
	else
	{
		selectRc5Code = 63;									// reset the variable
	}
	
	switch(selectRc5Code)									// selecting colors
	{
		case selectWhite:
		rc5Code = white;
		break;
				
		case selectRed:
		rc5Code = red;
		break;
				
		case selectGreen:
		rc5Code = green;
		break;
				
		case selectBlue:
		rc5Code = blue;
		break;
				
		case selectYellow:
		rc5Code = yellow;
		break;
				
		case selectPurple:
		rc5Code = purple;
		break;
				
		case selectTurquoise:
		rc5Code = turquoise;
		break;
				
		case selectBlack:
		rc5Code = black;
		break;
				
		default:
		rc5Code = 0b000000;
		break;
	}
	
	
	for(i = rc5CodeMSB; i>=rc5CodeLSB; i--)					// For loop to write the new RC5 code in to the RC5 code
	{
		if(rc5Code & rc5CodeBits)							// checks if the RC5 code bit is 1
		{
			rc5Code |= ((uint8_t)1 << i);					// writes the bit (1) in to RC5 code  
		}
		else												// else the RC5 code bit is 0
		{
			rc5Code &= ~((uint8_t)1 << i);					// writes the bit (0) in to RC5 code
		}
		rc5CodeBits = (rc5CodeBits >> (uint8_t)1);			// goes to the next bit of the RC5 code
	}
	
return rc5Code;												// returns the generated RC5 code

}



void generateRc5Command()									// Method to generate the RC5 Command
{
	uint16_t rc5StartBits =		 0x2000;					// RC5 Start bit mask
	uint16_t rc5FieldBits =      0x1000;					// RC5 Field bit mask
	uint16_t rc5ToggleBits =     0x800;						// RC5 Toggle bit mask
	uint16_t rc5AddressBits =    0x400;						// RC5 Address bits mask
	uint16_t rc5CodeBits =       0x20;						// RC5 Code bits mask


	for(i = rc5StartMSB; i>=rc5StartLSB; i--)				// For loop to write the RC5 start bit in to the RC5 command
	{
		if(rc5Start & rc5StartBits)							// checks if the RC5 start bit is 1
		{
			rc5Command |= ((uint16_t)1 << i);				// writes the bit (1) in to RC5 command
		}
	}
	
	
	for(i = rc5FieldMSB; i>=rc5FieldLSB; i--)				// For loop to write the RC5 field bit in to the RC5 command
	{
		
		if(rc5Field & rc5FieldBits)							// checks if the RC5 field bit is 1
		{
			rc5Command |= ((uint16_t)1 << i);				// writes the bit (1) in to RC5 command
		}
		else												// else the RC5 field bit is 0
		{
			rc5Command &= ~((uint16_t)1 << i);				// writes the bit (0) in to RC5 command
		}
	}
	
	
	for(i = rc5ToggleMSB; i>=rc5ToggleLSB; i--)				// For loop to write the RC5 toggle bit in to the RC5 command
	{
		
		if(!(rc5Toggle & rc5ToggleBits))					// checks if the RC5 toggle bit is 0
		{
			rc5Command &= ~((uint16_t)1 << i);				// writes the bit (0) in to RC5 command
			rc5Toggle |= ((uint16_t)1 << i);				// writes the bit (1) in to RC5 toggle
		}
		else                                                // else the RC5 toggle bit is 1
		{
			rc5Command |= ((uint16_t)1 << i);				// writes the bit (1) in to RC5 command
			rc5Toggle &= ~((uint16_t)1 << i);				// writes the bit (0) in to RC5 toggle
		}
	}
	
	
	for(i = rc5AddressMSB; i>=rc5AddressLSB; i--)			// For loop to write the RC5 address bits in to the RC5 command
	{
		if(rc5Address & rc5AddressBits)						// checks if the RC5 address bit is 1
		{
			rc5Command |= ((uint16_t)1 << i);				// writes the bit (1) in to RC5 command
		}
		else                                                // else the RC5 address bit is 0
		{
			rc5Command &= ~((uint16_t)1 << i);				// writes the bit (0) in to RC5 command
		}
		rc5AddressBits = (rc5AddressBits >> (uint16_t)1);	// goes to the next bit of the RC5 address
	}
	
	
	for(i = rc5CodeMSB; i>=rc5CodeLSB; i--)					// For loop to write the RC5 code bits in to the RC5 command
	{
		if(rc5Code & rc5CodeBits)							// checks if the RC5 code bit is 1
		{
			rc5Command |= ((uint16_t)1 << i);				// writes the bit (1) in to RC5 command
		}
		else                                                // else the RC5 code bit is 0
		{
			rc5Command &= ~((uint16_t)1 << i);				// writes the bit (0) in to RC5 command
		}
		rc5CodeBits = (rc5CodeBits >> (uint16_t)1);			// goes to the next bit of the RC5 code
	}	
}



int main(void)												// Main method
{


  DDRD |= (1 << PORTD7);									// Set port D 7 as output

  TCCR0A = (1 << WGM01);									// Set Timer normal operating mode
  
  TCCR0B = (1 << CS00);										// Set Timer clock source 1
  
  OCR0A = 1-1;											// Set the compare match value
 
  sei();													// Enable global interrupts
   
  while(1)													// Endless loop
  {	 
	  generateRc5Code(rc5Code);								// generates the RC5 code
	  generateRc5Command();									// generates the RC5 Command
	  
	  while(rc5CommandSent == 0)							// while RC5 Command is send
	  {
		 TIMSK0 |= (1<<OCIE0A);								// Enable timer 0 compare match interrupt
	  }
	  TIMSK0 &= ~(1<<OCIE0A);								// Disable timer 0 campare match interrupt
	 _delay_ms(1000);										// delay 1 s
	  rc5CommandSent = 0;									// Reset the variable to send the next RC5 frame
  }
}


ISR (TIMER0_COMPA_vect)														// ISR Timer 0 compare match
{
	switch(compareMatchCounter)												// checks the counts of the timer 0 compare match ISR
	{
		case 63:															// ~64 = (889 us / (1/36000kHz)) * 2
			if(rc5BitTransmit == 2)											// half bit transmit timing
			{	
				rc5BitTransmit = 0;											// reset the half bit transmit timing variable
				rc5Commandmask = (rc5Commandmask >> (uint16_t)1);			// goes to the next bit of the RC5 command
			}
		rc5BitTransmit++;													// increments the half bit transmit timing
		break;																// break switch statement
	}


	if(rc5Command & rc5Commandmask)											// checks if the RC5 command bit is 1
	{
		if((rc5Command & rc5Commandmask) && (rc5BitTransmit == 1))			// RC5 command bit is 1 and transmit halfbit low
		{
			PORTD &= ~(1 << PD7);											// Set the port d 7 low
		}
		if((rc5Command & rc5Commandmask) && (rc5BitTransmit == 2))			// RC5 command bit is 1 and transmit halfbit high
		{
			PORTD ^= (1 << PD7);											// Toggle the port d 7 high/low
		}
	}

	else if(!(rc5Command & rc5Commandmask))									// checks if the RC5 command bit is 0
	{
		if(!(rc5Command & rc5Commandmask) && (rc5BitTransmit == 1))			// RC5 command bit is 1 and transmit halfbit high
		{
			PORTD ^= (1 << PD7);											// Toggle the port d 7 high/low
		}
		if(!(rc5Command & rc5Commandmask) && (rc5BitTransmit == 2))			// RC5 command bit is 1 and transmit halfbit low
		{
			PORTD &= ~(1 << PD7);											// Set the port d 7 low
		}
	}


	if(compareMatchCounter <= 63)											 
	{
		compareMatchCounter++;												// increments the timer 0 compare match ISR counter
	}
			
	if(compareMatchCounter == 64)											
	{
		compareMatchCounter = 0;											// resets the timer 0 compare match ISR counter
	}


	if(rc5Commandmask == 0x0)												 
	{
		rc5Commandmask = 0x2000;											// resets the RC5 command mask
		rc5CommandSent = 1;													// the RC5 command is complete send
	}
}
