/*
 *  RC5 Receiver v0.1
 *
 *  Created: 05.09.2014 22:53:29
 *  Author: Fabio
 *  License: Open source
 */  


#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdbool.h>
#define F_CPU 8000000UL

#include <util/delay.h>

#define nop() \
 asm volatile ("nop")



volatile uint16_t rc5Command =     0b00000000000000;							// Stores the entire RC5 Command
volatile uint16_t rc5Commandmask = 0x2000;										// RC5 Command mask
volatile uint8_t rc5Bits = 13;													// Stores the actual bit to write it into rc5command

volatile uint8_t externalInterrupt = 0;											// Stores the number of the INT0
volatile uint16_t fail = 0;														// variable to evaluate the RC5command (correct / wrong)

volatile uint16_t tcnt1 = 0;													// Stores the Timer1 counts
volatile uint16_t timerPreload = 0;												// Stores Timer1 preload value
volatile uint8_t sampleBits = 0;												// Stores sampled bits count since MSB address


const uint16_t halfBitPeriodMin = 5120;											// Min half bit Timer1 counts (0.000640s / (1 / 8000000)s) = 5120
const uint16_t halfBitPeriodMax = 9120;											// Max half bit Timer1 counts (0.001140s / (1 / 8000000)s) = 9120       

const uint16_t fullBitPeriodMin = 10720;										// Min full bit Timer1 counts (0.001340s / (1 / 8000000)s) = 10720        
const uint16_t fullBitPeriodMax = 17760;										// Max full bit Timer1 counts (0.002220s / (1 / 8000000)s) = 17760


volatile uint32_t bits16 = 65536;												// Timer1 max counts

volatile uint8_t sampleData = 0;												// set to sample date with Timer1 OVF

volatile uint8_t dataSampled = 1;												// set if the whole date is sampled






int main(void)																	// Main method
{	

	DDRD &= ~(1 << PORTD2);														// set IR Receiver as output	
	DDRD |= (1 << PORTD6);														// set green led as output
	PORTD |= (1 << PORTD2);														// set IR Receiver (pull up resistor)
	PORTD |= (1 << PORTD6);														// set green led high (off)

	DDRB |= ((1 << PORTB1) | (1 << PORTB2));									// set blue / red led as output
	PORTB |= ((1 << PORTB1) | (1 << PORTB2));									// set blue / red led high (off)


	EICRA |= (1 << ISC00);														// external interrupt on any logical edge

	EIMSK |= (1 << INT0);														// enable external interrupt (INT0)

	sei();																		// Enable global interrupts

  
	while(1)																	// Endless loop
	{

		if (rc5Command == 0x3F)													// rc5 code 0b111111
		{
			/*white*/
			PORTB &= ~((1 << PORTB1) | (1 << PORTB2));
			PORTD &= ~(1 << PORTD6);
		}
		else if (rc5Command == 0x3E)
		{
			/*red*/
			PORTB &= ~(1 << PORTB2);
		}
		else if (rc5Command == 0x3D)
		{
			/*green*/
			PORTD &= ~(1 << PORTD6);
		}
		else if (rc5Command == 0x3C)
		{
			/*blue*/
			PORTB &= ~(1 << PORTB1);
		}
		else if (rc5Command == 0x3B)
		{
			/*yellow*/
			PORTB &= ~(1 << PORTB2);
			PORTD &= ~(1 << PORTD6);
		}
		else if (rc5Command == 0x3A)
		{
			/*purple*/
			PORTB &= ~(1 << PORTB2);
			PORTB &= ~(1 << PORTB1);
		}
		else if (rc5Command == 0x39)
		{
			/*turquoise*/
			PORTD &= ~(1 << PORTD6);
			PORTB &= ~(1 << PORTB1);
		}
		else if (rc5Command == 0x38)											// rc5 code 0b111000
		{
			/*black*/
			PORTB |= ((1 << PORTB1) | (1 << PORTB2));
			PORTD |= (1 << PORTD6);
		}
		
	}
	
}





ISR(INT0_vect)																	// ISR external interrupt 0
{
	externalInterrupt++;														// counts the externals interrupts

	if(fail == 0)																// no error until now
	{

		if (externalInterrupt == 1)												// first external interrupt
		{
			tcnt1 = 0;															// clear variable
			TCNT1 = 0;															// clear TCNT1 counts
			rc5Bits = 13;														// Stores the actual bit to write it into rc5command
			TIMSK1 &= ~(1 << TOIE1);											// disable timer1 ovf interrupt
			TCCR1B |= (1 << CS10);												// starts the timer1 with clock source (start timer1)
    	
			rc5Command |= ((uint16_t)1 << rc5Bits);								// stores the start bit into rc5comman
			rc5Bits--;															// decrements the actual bit to write it into rc5command
			//return;															// i am not sure if used in this case??
		}
		else if (externalInterrupt == 2)										// second external interrupt
		{
			TCCR1B &= ~((1 << CS12) | (1 << CS11) | (1 << CS10));				// Disconnect Timer1 clock source (stop timer1)
			tcnt1 = TCNT1;														// Store actual Timer1 counts into tcnt1 (header time of the frame)
			TCNT1 = 0;															// clear TCNT1 counts
									 	                            
			if (tcnt1 > halfBitPeriodMin && tcnt1 < halfBitPeriodMax)			// checks the header timing is between the half bit timing          
			{	/*thats the field bit 1*/
				rc5Command |= ((uint16_t)1 << rc5Bits);							// stores the field bit if it is 1 into rc5command
				rc5Bits--;														// decrements the actual bit to write it into rc5command
				//return;														// i am not sure if used in this case??
			}
			else if (tcnt1 > fullBitPeriodMin && tcnt1 < fullBitPeriodMax)		// checks the header timing is between the full bit timing 
			{	/*thats the field bit 0*/
				EICRA &= ~((1 << ISC01) | (1 << ISC00));						
				EIMSK &= ~(1 << INT0);											// disable the external interrupt
				rc5Command &= ~(1 << rc5Bits);									// stores the field bit if it is 0 into rc5command
				rc5Bits--;														// decrements the actual bit to write it into rc5command
      			
				for (int i = 0 ; i < tcnt1 * 0.75; i++)							// delay 3/4 bit time
				{   /*mach was oder chills eifach*/
					asm volatile ("nop");										// no operation
				}

				if (PIND & (1 << PORTD2))										// if IR receiver input is high
				{	/*thats the toggle bit 1*/
					rc5Command |= ((uint16_t)1 << rc5Bits);						// stores the field bit if it is 1 into rc5command
					rc5Bits--;													// decrements the actual bit to write it into rc5command
				}
				else if (!(PIND & (1 << PORTD2)))								// if IR receiver input is low
				{	/*thats the toggle bit 0*/
					rc5Command &= ~((uint16_t)1 << rc5Bits);					// stores the field bit if it is 0 into rc5command
					rc5Bits--;													// decrements the actual bit to write it into rc5command
				}

				timerPreload = (bits16 - tcnt1);								// calculate timer preloader if field bit is 0
				TCNT1 = timerPreload;											// set timer1 register to generate interrupt each full bit time
				TIMSK1 |= (1 << TOIE1);											// enable timer1 ovf interrupt
				TCCR1B |= (1 << CS10);											// starts the timer1 with clock source (start timer1) 
      
				sampleData = 1;													// set to sample date with Timer1 OVF  
				dataSampled = 0;												// set if the whole date is sampled  
				//return;														// i am not sure if used in this case??
			}
			else if (tcnt1 < halfBitPeriodMin || tcnt1 > fullBitPeriodMax ||
					(tcnt1 > halfBitPeriodMax && tcnt1 < fullBitPeriodMin))		// checks the header timing its not between the half or full bit time then its a fail
			{	/*Invalid bit time */
				EICRA &= ~((1 << ISC01) | (1 << ISC00));						// INT0 normal port operation (disable INT0)
				EIMSK &= ~(1 << INT0);											// disable the external interrupt
				TIMSK1 |= (1 << TOIE1);											// enable timer1 ovf interrupt
				TCCR1B |= (1 << CS10);											// starts the timer1 with clock source (start timer1)
				timerPreload = (bits16 - fullBitPeriodMax);						// calculate timer preloader
				TCNT1 = timerPreload;											// set timer1 register to generate interrupt each full bit time
      
				fail = 1;														// error has occurred
				sampleData = 1;													// set to sample date with Timer1 OVF
				dataSampled = 0;												// set if the whole date is sampled												
				//return;														// i am not sure if used in this case??
			}
		}
		else if (externalInterrupt == 3)										// third external interrupt		
		{	/*thats the toggle bit*/

			EICRA &= ~((1 << ISC01) | (1 << ISC00));							// INT0 normal port operation (disable INT0)
			EIMSK &= ~(1 << INT0);												// disable the external interrupt
    
			for (int i = 0 ; i < tcnt1 * 2 * 0.75; i++)							// delay 3/4 bit time
			{   /*mach was oder chills eifach*/
				asm volatile ("nop");											// no operation
			}

			if (PIND & (1 << PORTD2))											// if IR receiver input is high
			{	/*thats the toggle bit 1*/
				rc5Command |= ((uint16_t)1 << rc5Bits);							// stores the field bit if it is 0 into rc5command
				rc5Bits--;														// decrements the actual bit to write it into rc5command
			}
			else if (!(PIND & (1 << PORTD2)))									// if IR receiver input is low
			{	/*thats the toggle bit 0*/
				rc5Command &= ~((uint16_t)1 << rc5Bits);						// stores the field bit if it is 0 into rc5command
				rc5Bits--;														// decrements the actual bit to write it into rc5command
			}

			timerPreload = (bits16 - (tcnt1 * 2));								// calculate timer preloader if field bit is 1					
			TCNT1 = timerPreload;												// set timer1 register to generate interrupt each full bit time
			TIMSK1 |= (1 << TOIE1);												// enable timer1 ovf interrupt
			TCCR1B |= (1 << CS10);												// starts the timer1 with clock source (start timer1)
      
			sampleData = 1;														// set to sample date with Timer1 OVF
			dataSampled = 0;													// set if the whole date is sampled
			//return;															// i am not sure if used in this case??

		}
	}
}


ISR(TIMER1_OVF_vect)															// ISR timer1 OVF interrupt
{

	if(fail == 0)																// no error until now
	{
		EICRA &= ~((1 << ISC01) | (1 << ISC00));								// INT0 normal port operation (disable INT0)
		EIMSK &= ~(1 << INT0);													// disable the external interrupt
		
		if (sampleBits == 11)													// all data of rc5commands are sampled
		{ 
			if (PIND & (1 << PORTD2))											// and idle state is present
			{	/*Idle State = HIGH*/
				dataSampled = 1;												// set if the whole date is sampled
				fail = 0;														// reset fail variable
				tcnt1 = 0;														// reset tcnt1 variable
				externalInterrupt = 0;											// reset external interrupt
				sampleData = 0;													// set to sample date with Timer1 OVF
				sampleBits = 0;													// reset sampled bits variable
				rc5Bits = 13;													// Stores the actual bit to write it into rc5command
				timerPreload = 0;												// reset timer preloader variable												
				TCNT1 = 0;														// reset TCNT1 register
				TCCR1B &= ~((1 << CS12) | (1 << CS11) | (1 << CS10));			// disconnect the clock source of timer1 	
				TIMSK1 &= ~(1 << TOIE1);										// disable timer1 ovf interrupt									
				EICRA |= (1 << ISC00);											// external interrupt on any logical edge
				EIMSK |= (1 << INT0);											// enable external interrupt (INT0)						
			}
		}

		if(sampleData == 1 && dataSampled == 0)									// sample the date since MSB address bit
		{
			if (PIND & (1 << PORTD2))											// if IR receiver input is high
			{	/*Sample Data 1*/
				rc5Command |= ((uint16_t)1 << rc5Bits);							// stores the bits into rc5command (1)
				rc5Bits--;														// decrements the actual bit to write it into rc5command
				sampleBits++;													// counts the sampled bits
			}
			else if (!(PIND & (1 << PORTD2)))									// if IR receiver input is low
			{	/*Sample Data 0*/
				rc5Command &= ~((uint16_t)1 << rc5Bits);						// stores the bits into rc5command (0)
				rc5Bits--;														// decrements the actual bit to write it into rc5command
				sampleBits++;													// counts the sampled bits
			}
		}
	}
	else if (fail == 1)
	{
		EICRA &= ~((1 << ISC01) | (1 << ISC00));
		EIMSK &= ~(1 << INT0);
		
		if (sampleBits == 11)
		{ 
			if (PIND & (1 << PORTD2))
			{	/*Idle State = HIGH*/
				fail = 0;
				tcnt1 = 0;
				dataSampled = 1;
				externalInterrupt = 0;
				sampleData = 0;
				sampleBits = 0;
				rc5Bits = 13;
				timerPreload = 0;
				TCNT1 = 0;
                TCCR1B &= ~((1 << CS12) | (1 << CS11) | (1 << CS10));
                TIMSK1 &= ~(1 << TOIE1);
                EICRA |= (1 << ISC00);
                EIMSK |= (1 << INT0);
			}
		}
    
		if(sampleData == 1 && dataSampled == 0)
		{
			if (PIND & (1 << PORTD2))
			{	/*Sample Data 1*/
				rc5Command |= ((uint16_t)1 << rc5Bits);
				rc5Bits--;
				sampleBits++; 
			}
			else if (!(PIND & (1 << PORTD2)))
			{	/*Sample Data 0*/
				rc5Command &= ~((uint16_t)1 << rc5Bits);
				rc5Bits--;
				sampleBits++; 
			}
		}  
	}

  TCNT1 = timerPreload;															// set TCNT1 register with the timer preload value
  
}